/**
 * Simple QtOpenGL Sample
 * tuomo.hirvonen @ digia 2010
 *
 */


#include "glwindow.h"


    // define our cube
const GLfloat cube_vertices[] = { -1.0f,-1.0f,1.0f, 1.0f,-1.0f,1.0f, 1.0f,1.0f,1.0f, -1.0f,1.0f,1.0f,
                                  -1.0f,-1.0f,-1.0f, 1.0f,-1.0f,-1.0f, 1.0f,1.0f,-1.0f, -1.0f,1.0f,-1.0f };
const GLushort cube_indices[]= {0,1,2, 0,2,3, 1,5,6, 1,6,2, 4,0,3, 4,3,7, 5,4,7, 5,7,6, 4,5,1, 4,1,0, 2,6,7, 2,7,3 };



const char *sample_vertex_shader_src =
    "attribute highp vec4 vertex;\n"
    "uniform mediump mat4 matrix;\n"
    "uniform mediump mat4 proj;\n"
    "varying highp vec4 frag_pos;\n"
    "void main(void)\n"
        "{\n"
        "    frag_pos = matrix * vertex;\n"
        "    gl_Position = proj*frag_pos;\n"
        "}\n";

const char *sample_fragment_shader_src =
    "varying highp vec4 frag_pos;\n"
    "void main(void)\n"
    "{\n"
    "    mediump float light = 2.0 / -(frag_pos.z);\n"
    "    gl_FragColor = vec4(light, light, light, 0.5);\n"
    "}\n";


GLWindow::GLWindow(QWidget *parent) : QGLWidget(parent) {
        // Set some Qt-attributes to make the rendering faster.
    setAutoFillBackground( false );
    setAttribute (Qt::WA_OpaquePaintEvent);
    setAttribute( Qt::WA_NoSystemBackground );
    setAttribute( Qt::WA_NativeWindow );
    setAttribute( Qt::WA_PaintOnScreen, true );
    setAttribute( Qt::WA_StyledBackground,false);
    setAutoBufferSwap(false);

        // Startup the timer which will call updateGL as fast as it can.
    QTimer *timer = new QTimer(this);
    QObject::connect(timer, SIGNAL(timeout()), this, SLOT(updateGL()));
    timer->start();
}


GLWindow::~GLWindow() {

}


void GLWindow::resizeGL(int width, int height) {
        // Reset the Gl viewport for current resolution.
    glViewport(0,0, width, height);
};



void GLWindow::paintGL() {
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);                             // Clear the background and depth-buffer for this frame

    GLint vertexAttr = m_sampleProgram.attributeLocation("vertex");
    GLint matrixAttr = m_sampleProgram.uniformLocation("matrix");
    GLint projAttr = m_sampleProgram.uniformLocation("proj");


    m_sampleProgram.bind();
    m_sampleProgram.enableAttributeArray(vertexAttr);
    m_sampleProgram.setAttributeArray(vertexAttr, cube_vertices, 3 );               // Set vertexarray to the shader

    QMatrix4x4 projection;
    projection.perspective(45.0f, (float)width()/(float)height(), 0.1f, 20.0f );    // Use perspective projection
   /*
        // output the projectionmatrix
    qDebug() << "const float *projectionMatrix = {";
    for (int f=0; f<4; f++)
        qDebug() << projection.column(f).x() << "," << projection.column(f).y() << "," << projection.column(f).z() << "," << projection.column(f).w() << ",";
    qDebug() << "};";
*/


    m_sampleProgram.setUniformValue( projAttr, projection );                        // Set projection to the shader




        // Orientation for the cube we are about to render
    QMatrix4x4 orientation;
    orientation.setToIdentity();
    orientation.translate(0.0f, 0.0f, -4.5f );                                      // 5 units away from camera
    static float ang = 0.0f;
    ang+=0.1f;
    orientation.rotate(ang, 1.0f, 0.0f, 0.0f );
    orientation.rotate(ang*0.7f, 0.0f, 1.0f, 0.0f );
    orientation.rotate(ang*1.4f, 0.0f, 0.0f, 1.0f );
    m_sampleProgram.setUniformValue( matrixAttr, orientation );                     // Set orientation to the shader

    glDrawElements(GL_TRIANGLES, 6*2*3, GL_UNSIGNED_SHORT, cube_indices );          // draw the cube.


    m_sampleProgram.disableAttributeArray(vertexAttr);                              // Disable the program
    m_sampleProgram.release();

    swapBuffers();                                                                  // Swap buffers manually since automatic swapping was disabled in a constructor.
};



void GLWindow::initializeGL() {

    QGLShader *vertex_shader = new QGLShader(QGLShader::Vertex, this);              // Create and compile the vertex-shader
    vertex_shader->compileSourceCode(sample_vertex_shader_src);

    QGLShader *fragment_shader = new QGLShader(QGLShader::Fragment, this);          // Create and compile the fragment-shader
    fragment_shader->compileSourceCode(sample_fragment_shader_src);

    m_sampleProgram.addShader(vertex_shader);                                       // Add vertex-shader to our program
    m_sampleProgram.addShader(fragment_shader);                                     // Add fragment-shader to our program
    m_sampleProgram.link();                                                         // Link our program. Its now ready for use.

    glFrontFace(GL_CCW);
    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);
    glClearColor(0.1f, 0.4f, 0.1f, 1.0f);                                           // Clear the background with dark-green
};



